package edu.ubc.cpsc.googlex;

import java.io.Serializable;
import com.google.soap.search.*;

/**
 * A serializable wrapper for search results.
 * 
 * @author Steve Wolfman
 */
public class GoogleSearchResultWrapper implements Serializable
{
    private GoogleSearchDirectoryCategoryWrapper[] myDirectoryCategories;
    private GoogleSearchResultElementWrapper[] myResultElements;

    private boolean myDocumentFiltering;
    private int myEndIndex;
    private int myEstimatedTotalResultsCount;
    private boolean myEstimateIsExact;
    private String mySearchComments;
    private String mySearchQuery;
    private double mySearchTime;
    private String mySearchTips;
    private int myStartIndex;

    /**
     * Generate a wrapper to represent the given result
     * 
     * @param wrapped a non-null search result to wrap
     */
    public GoogleSearchResultWrapper( GoogleSearchResult wrapped )
    {
        if( wrapped == null )
            throw new IllegalArgumentException( "wrapped is null" );

        this.myDirectoryCategories = new GoogleSearchDirectoryCategoryWrapper[ wrapped
                .getDirectoryCategories().length ];
        for( int i = 0; i < this.myDirectoryCategories.length; i++ )
            this.myDirectoryCategories[ i ] = new GoogleSearchDirectoryCategoryWrapper(
                    wrapped.getDirectoryCategories()[ i ] );

        this.myResultElements = new GoogleSearchResultElementWrapper[ wrapped
                .getResultElements().length ];
        for( int i = 0; i < this.myResultElements.length; i++ )
            this.myResultElements[ i ] = new GoogleSearchResultElementWrapper(
                    wrapped.getResultElements()[ i ] );

        this.myDocumentFiltering = wrapped.getDocumentFiltering();
        this.myEndIndex = wrapped.getEndIndex();
        this.myEstimatedTotalResultsCount = wrapped
                .getEstimatedTotalResultsCount();
        this.myEstimateIsExact = wrapped.getEstimateIsExact();
        this.mySearchComments = wrapped.getSearchComments();
        this.mySearchQuery = wrapped.getSearchQuery();
        this.mySearchTime = wrapped.getSearchTime();
        this.mySearchTips = wrapped.getSearchTips();
        this.myStartIndex = wrapped.getStartIndex();
    }

    /**
     * Generates the wrapped GoogleSearchResult. Note, this returns a
     * <em>new</em> GoogleSearchResult each time.
     * 
     * @return a GoogleSearchResult representing the wrapped result
     */
    public GoogleSearchResult unwrap()
    {
        GoogleSearchResult wrapped = new GoogleSearchResult();

        GoogleSearchDirectoryCategory[] categories = new GoogleSearchDirectoryCategory[ this.myDirectoryCategories.length ];
        for( int i = 0; i < categories.length; i++ )
            categories[ i ] = this.myDirectoryCategories[ i ].unwrap();
        wrapped.setDirectoryCategories( categories );

        GoogleSearchResultElement[] results = new GoogleSearchResultElement[ this.myResultElements.length ];
        for( int i = 0; i < results.length; i++ )
            results[ i ] = this.myResultElements[ i ].unwrap();
        wrapped.setResultElements( results );

        wrapped.setDocumentFiltering( this.myDocumentFiltering );
        wrapped.setEndIndex( this.myEndIndex );
        wrapped
                .setEstimatedTotalResultsCount( this.myEstimatedTotalResultsCount );
        wrapped.setEstimateIsExact( this.myEstimateIsExact );
        wrapped.setSearchComments( this.mySearchComments );
        wrapped.setSearchQuery( this.mySearchQuery );
        wrapped.setSearchTime( this.mySearchTime );
        wrapped.setSearchTips( this.mySearchTips );
        wrapped.setStartIndex( this.myStartIndex );

        return wrapped;
    }
}

/**
 * A serializable wrapper for search directory categories.
 * 
 * @author Steve Wolfman
 */

class GoogleSearchDirectoryCategoryWrapper implements Serializable
{
    private String myFullViewableName;
    private String mySpecialEncoding;

    /**
     * Generate a wrapper to represent the given category
     * 
     * @param wrapped a non-null category to wrap
     */
    public GoogleSearchDirectoryCategoryWrapper(
            GoogleSearchDirectoryCategory wrapped )
    {
        if( wrapped == null )
            throw new IllegalArgumentException( "wrapped is null" );

        this.myFullViewableName = wrapped.getFullViewableName();
        this.mySpecialEncoding = wrapped.getSpecialEncoding();
    }

    /**
     * Generates the wrapped GoogleSearchDirectoryCategory. Note, this returns a
     * <em>new</em> GoogleSearchDirectoryCategory each time.
     * 
     * @return a GoogleSearchDirectoryCategory representing the wrapped category
     */
    public GoogleSearchDirectoryCategory unwrap()
    {
        GoogleSearchDirectoryCategory wrapped = new GoogleSearchDirectoryCategory();

        wrapped.setFullViewableName( this.myFullViewableName );
        wrapped.setSpecialEncoding( this.mySpecialEncoding );

        return wrapped;
    }
}

/**
 * A serializable wrapper for search result elements.
 *
 * @author Steve Wolfman
 */

class GoogleSearchResultElementWrapper implements Serializable
{
    private GoogleSearchDirectoryCategoryWrapper myDirectoryCategory;

    private String myCachedSize;
    private String myDirectoryTitle;
    private String myHostName;
    private boolean myRelatedInformationPresent;
    private String mySnippet;
    private String mySummary;
    private String myTitle;
    private String myURL;

    /**
     * Generate a wrapper to represent the given result element
     * 
     * @param wrapped a non-null result element to wrap
     */
    public GoogleSearchResultElementWrapper( GoogleSearchResultElement wrapped )
    {
        if( wrapped == null )
            throw new IllegalArgumentException( "wrapped is null" );

        this.myDirectoryCategory = new GoogleSearchDirectoryCategoryWrapper(
                wrapped.getDirectoryCategory() );

        this.myCachedSize = wrapped.getCachedSize();
        this.myDirectoryTitle = wrapped.getDirectoryTitle();
        this.myHostName = wrapped.getHostName();
        this.myRelatedInformationPresent = wrapped
                .getRelatedInformationPresent();
        this.mySnippet = wrapped.getSnippet();
        this.mySummary = wrapped.getSummary();
        this.myTitle = wrapped.getTitle();
        this.myURL = wrapped.getURL();
    }

    /**
     * Generates the wrapped GoogleSearchResultElement. Note, this returns a
     * <em>new</em> GoogleSearchResultElement each time.
     * 
     * @return a GoogleSearchResultElement representing the wrapped result
     *         element
     */
    public GoogleSearchResultElement unwrap()
    {
        GoogleSearchResultElement wrapped = new GoogleSearchResultElement();

        wrapped.setDirectoryCategory( this.myDirectoryCategory.unwrap() );

        wrapped.setCachedSize( this.myCachedSize );
        wrapped.setDirectoryTitle( this.myDirectoryTitle );
        wrapped.setHostName( this.myHostName );
        wrapped.setRelatedInformationPresent( this.myRelatedInformationPresent );
        wrapped.setSnippet( this.mySnippet );
        wrapped.setSummary( this.mySummary );
        wrapped.setTitle( this.myTitle );
        wrapped.setURL( this.myURL );

        return wrapped;
    }
}