package edu.ubc.cpsc.googlex;

import java.io.*;
import java.util.*;
import com.google.soap.search.*;

/**
 * A "Google" Search that actually just regurgitates stored search results. It
 * uses a log (generally generated by a LoggingGoogleSearch) comprised of a
 * series of serialized GoogleSearchResultWrappers. New searches just index into
 * these wrappers. Searches are decomposed into terms by tokenizing into words
 * (respecting double-quote-delimited sequences) and looking up these words. A
 * search query that was never logged always returns the same result, which
 * represents an unfound search.
 * <p>
 * For simplicity, although this is certainly not the best design, the log is
 * stored as a static member. All search instances fetch from the same log, and
 * new instances can be created without knowing about or specifying the log.
 * (Thus, the constructor is as similar as possible to the plain GoogleSearch
 * constructor.)
 * <p>
 * Note that only actual searches work. Cache requests and spelling requests are
 * unsupported.
 * <p>
 * Finally, note that searches are stored ONLY by query, and even the query is
 * normalized; so, results will ignore startIndex, maxResults, filtering, ...
 * and return whatever was stored.
 * 
 * @author Steve Wolfman
 */
public class LoggedGoogleSearch extends GoogleSearch
{
    /**
     * A mapping between normalized queries and their associated search results.
     */
    private static Map<String, GoogleSearchResult> ourCachedSearches = new HashMap<String, GoogleSearchResult>();

    /**
     * A static initializer, which is like a constructor but for the class
     * rather than any particular instance. In this case, initialize the cache
     * to use our hard-coded values.
     */
    static
    {
        initializeSearchCache();
    }

    /**
     * A single log stream shared by all LoggedGoogleSearch instances. If the
     * log is uninitialized, logged searches simply fail.
     */
    private static ObjectInputStream ourLog = null;

    /**
     * Default log file to use if no log is set.
     * 
     * @see #setLog(String)
     */
    public static final String DEFAULT_LOG_FILE = "test.log";

    /**
     * Initialize the log to its default value.
     */
    static
    {
        // To make this simpler for students, the log is initialized to a
        // default automatically.
        setLog( DEFAULT_LOG_FILE );
    }

    /**
     * Fill the cached search table with everything in the current log. (Empties
     * the table before filling with the current log.)
     */
    private static void initializeSearchCache()
    {
        // Clear the cached searches.
        ourCachedSearches.clear();

        // If the log is null, there's nothing more to do.
        if( ourLog == null )
            return;

        // Loop "forever" reading and processing GoogleSearchResultWrappers
        // until end of file causes an EOFException or IOException.
        // (ObjectInputStreams seem to generally give IOExceptions rather than
        // EOFExceptions.)
        //
        // Note: objects of unknown or inappropriate type are ignored WITHOUT
        // errors.
        //
        // Note: loop exit is at breaks.
        while( true )
        {
            Object o = null;
            try
            {
                o = ourLog.readObject();
            }
            catch( EOFException e )
            {
                // LOOP BROKEN HERE
                // Reached end of file, break out of the loop.
                break;
            }
            catch( ClassNotFoundException e )
            {
                // LOOP CONTINUED HERE
                // Ignore this object of unknown type.
                continue;
            }
            catch( IOException e )
            {
                // LOOP BROKEN HERE
                // Reached end of file, break out of the loop.
                break;
            }

            if( !( o instanceof GoogleSearchResultWrapper ) )
                // LOOP CONTINUED HERE
                // Ignore this object of inappropriate type.
                continue;

            // All's well, transform the object to a GoogleSearchResult.
            GoogleSearchResultWrapper wrapper = (GoogleSearchResultWrapper) o;
            GoogleSearchResult result = wrapper.unwrap();

            // Then, store the result, indexed by its normalized query.
            String normalizedQuery = normalizeQuery( result.getSearchQuery() );
            ourCachedSearches.put( normalizedQuery, result );
        }
    }

    /**
     * Convencience method: creates an ObjectInputStream from the given name,
     * sets the log to that stream, and closes the log after setting the stream.
     * Also summarizes all exceptions (except runtime exceptions) as runtime
     * IllegalArgumentExceptions.
     * 
     * @param filename a non-null filename
     * 
     * @see #setLog(ObjectInputStream)
     */
    public static void setLog( String filename )
    {
        if( filename == null )
            throw new IllegalArgumentException( "parameter filename was null" );

        try
        {
            ObjectInputStream in = new ObjectInputStream( new FileInputStream(
                    filename ) );
            setLog( in );
            in.close();
        }
        catch( IOException e )
        {
            throw new IllegalArgumentException( "problem accessing file: "
                    + filename, e );
        }
    }

    /**
     * Attach all LoggedGoogleSearch instances to the given log. Note that the
     * log is be processed immediately and not again (at future queries). Reads
     * to the end of the log but does not close it. (Note: objects of unknown or
     * inappropriate type in the log are ignored. This is in part to support
     * forwards compatibility with future versions that handle cache requests
     * and spelling checks.)
     * 
     * @param log a log of serialized GoogleSearchResultWrappers, may be null
     * 
     * @see GoogleSearchResultWrapper
     */
    public static void setLog( ObjectInputStream log )
    {
        ourLog = log;

        initializeSearchCache();
    }

    /**
     * Returns the current (static) log or null if logging is disabled
     * 
     * @return the log
     */
    public static ObjectInputStream getLog()
    {
        return ourLog;
    }

    /**
     * Break a query into a sorted list of tokens, respecting
     * double-quote-delimited phrases.
     * <p>
     * The algorithm here is simple and fallible. It breaks the string along
     * whitespace. Any sequence of such tokens that starts with a token starting
     * with " is collected up to a token ending with " into a single phrase. "
     * in the middle of words are not handled.
     * <p>
     * Note, this is public largely for testing, but it could also be helpful
     * for other classes. If used in that way, it should probably eventually be
     * migrated to a more sensible location for a general utility.
     * 
     * @param query a non-null search query
     * @return a normalized search query: a sequence of words with no repeats in
     *         sorted order each separated by a single space with quoted phrases
     *         in which each word is also separated by a single space
     */
    public static String normalizeQuery( String query )
    {
        Scanner scanner = new Scanner( query.toLowerCase() );

        // A place to store the search terms as they're collected.
        // The fact that this is sorted ensures that order of terms is
        // immaterial.
        SortedSet<String> terms = new TreeSet<String>();

        // Tokenise the input.
        String phrase = null;
        while( scanner.hasNext() )
        {
            String token = scanner.next();
            if( phrase != null )
            {
                phrase += " " + token;
                if( token.endsWith( "\"" ) )
                {
                    terms.add( phrase );
                    phrase = null;
                }
            }
            else
            {
                // Check for the start of quoted phrases, but beware of
                // one word quoted phrases (which can be treated like any
                // other token).
                if( token.startsWith( "\"" ) && !token.endsWith( "\"" ) )
                    phrase = token;
                else
                    terms.add( token );
            }
        }

        // Google seems to treat unclosed quotes as closed.
        // So, this code does the same.
        if( phrase != null )
        {
            terms.add( phrase + "\"" );
            phrase = null;
        }

        // Construct a sorted query string from the set of terms.
        String result = "";
        if( terms.size() != 0 )
        {
            for( String s : terms )
                result += s + " ";
            result = result.substring( 0, result.length() - 1 );
        }
        return result;
    }

    /**
     * The most recently set query, in normalized form.
     */
    private String myNormalizedQuery = null;

    /**
     * Construct a logged google search that uses the shared (static) log as a
     * source of cached searches.
     */
    public LoggedGoogleSearch()
    {
        super();
    }

    /**
     * @see com.google.soap.search.GoogleSearch#setQueryString(java.lang.String)
     */
    public void setQueryString( String arg0 )
    {
        this.myNormalizedQuery = normalizeQuery( arg0 );

        super.setQueryString( arg0 );
    }

    /**
     * Unsupported in logged searches.
     * 
     * @see com.google.soap.search.GoogleSearch#doGetCachedPage(java.lang.String)
     */
    public byte[] doGetCachedPage( String arg0 ) throws GoogleSearchFault
    {
        throw new UnsupportedOperationException(
                "unsupported in logged searches" );
    }

    /**
     * Rather than performing an actual search, this tries to find the query in
     * a cached set of queries read from the shared (static) log.
     * 
     * @see LoggedGoogleSearch#LoggedGoogleSearch()
     * @see LoggedGoogleSearch#setLog
     * @see LoggedGoogleSearch#getLog
     * @see com.google.soap.search.GoogleSearch#doSearch()
     */
    public GoogleSearchResult doSearch() throws GoogleSearchFault
    {
        if( this.myNormalizedQuery == null )
            throw new GoogleSearchFault( "query was null" );

        GoogleSearchResult result;
        if( ourCachedSearches.containsKey( this.myNormalizedQuery ) )
            result = ourCachedSearches.get( this.myNormalizedQuery );
        else
            throw new GoogleSearchFault( "no logged entry exists for the query: " + this.myNormalizedQuery);
        return result;
    }

    /**
     * Unsupported in logged searches.
     * 
     * @see com.google.soap.search.GoogleSearch#doSpellingSuggestion(java.lang.String)
     */
    public String doSpellingSuggestion( String arg0 ) throws GoogleSearchFault
    {
        throw new UnsupportedOperationException(
                "unsupported in logged searches" );
    }
}