package edu.ubc.cpsc.googlex;

import com.google.soap.search.*;

import java.util.*;
import java.io.*;

/**
 * A Google Search that logs itself to a file for future use. Currently, only
 * logs GoogleSearchResults.
 * <p>
 * For simplicity, although this is certainly not the best design, the log is
 * stored as a static member. All search instances log to the same location, and
 * new instances can be created without knowing about or specifying the log.
 * (Thus, the constructor is as similar as possible to the plain GoogleSearch
 * constructor.)
 * <p>
 * Logs all searches as serialized GoogleSearchResultWrappers.
 * 
 * @author Steve Wolfman
 */
public class LoggingGoogleSearch extends GoogleSearch
{
    /**
     * A single log stream shared by all LoggingGoogleSearch instances. If the
     * log is uninitialized, logging searches simply do not log.
     */
    private static ObjectOutputStream ourLog = null;
    public static final String DEFAULT_LOG_FILE = "test.log";

    static
    {
        // To make this simpler for students, the log is initialized to a
        // default
        // automatically.
        //
        // WARNING: unpredictable results may occur, however, if students do not
        // close the log (by calling the finishLog method).
        setLog( DEFAULT_LOG_FILE );
    }

    /**
     * Set the current log. Note: new log entries will be appended to the log as
     * serialized GoogleSearchResultWrappers. Also note: this class will
     * <em>not</em> close the log.
     * 
     * @param log a valid object output stream for the log or null to disable
     *        logging
     * 
     * @see GoogleSearchResultWrapper
     */
    public static void setLog( ObjectOutputStream log )
    {
        ourLog = log;
    }

    /**
     * Convencience method: creates an ObjectOutputStream from the given name,
     * and sets the log to that stream. Also summarizes all exceptions (except
     * runtime exceptions) as runtime IllegalArgumentExceptions.
     * <p>
     * Finally, just appending from run to run does NOT seem to work. So, this
     * reads in all the objects on the input stream into a list, wipes the file,
     * and rewrites all the objects, leaving the stream ready for the next
     * object in sequence. <b>WARNING: </b> that means this could wipe your
     * whole file out if there's a problem reading some of the objects. (It'll
     * take the problem for end of file and ignore everything left, overwriting
     * it!)
     * <p>
     * Note: <em>still</em> does not close the log when done.
     * 
     * @see #finishLog()
     * 
     * @param filename a non-null filename
     */
    public static void setLog( String filename )
    {
        if( filename == null )
            throw new IllegalArgumentException( "parameter filename was null" );

        try
        {
            // Collect all the objects currently in the file into a list.
            List<Object> objects = new ArrayList<Object>();
            try
            {
                ObjectInputStream in = new ObjectInputStream(
                        new FileInputStream( filename ) );
                while( true )
                {
                    try
                    {
                        objects.add( in.readObject() );
                    }
                    catch( ClassNotFoundException e )
                    {
                        // Skip the unrecognized objects.
                        continue;
                    }
                    catch( IOException e )
                    {
                        // Quit on an IO/EOF Exception.
                        break;
                    }
                }
                in.close();
            }
            catch( IOException e )
            {
                // Ignore.
            }

            // Open the output stream for writing and write out the initial
            // objects.
            ObjectOutputStream out = new ObjectOutputStream(
                    new FileOutputStream( filename ) );
            for( Object o : objects )
                out.writeObject( o );

            // Set the log to this output stream.
            setLog( out );
        }
        catch( IOException e )
        {
            throw new IllegalArgumentException( "problem accessing file: "
                    + filename, e );
        }
    }

    /**
     * Returns the current (static) log or null if logging is disabled
     * 
     * @return the log
     */
    public static ObjectOutputStream getLog()
    {
        return ourLog;
    }

    /**
     * Convenience method that closes the current log, if any and if unclosed.
     * Note: hides any IOExceptions!
     */
    public static void finishLog()
    {
        if( ourLog != null )
        {
            try
            {
                ourLog.close();
            }
            catch( IOException e )
            {
                // Ignore the exception.
                System.out.println( "A problem occurred closing the log: "
                        + e.toString() );
                System.out.println( "Ignoring the problem and carrying on.." );
            }
        }
    }

    /**
     * Create a new logging search that uses the shared, static log.
     */
    public LoggingGoogleSearch()
    {
        super();
    }

    /**
     * Performs a standard search but also logs
     * 
     * @see com.google.soap.search.GoogleSearch#doSearch()
     */
    public GoogleSearchResult doSearch() throws GoogleSearchFault
    {
        GoogleSearchResult result = super.doSearch();

        if( ourLog != null )
        {
            GoogleSearchResultWrapper wrapper = new GoogleSearchResultWrapper(
                    result );
            try
            {
                ourLog.writeObject( wrapper );
            }
            catch( IOException e )
            {
                throw new GoogleSearchFault(
                        "Error trying to log the search result", e );
            }
        }

        return result;
    }

    /**
     * Unsupported in logging searches.
     * 
     * @see com.google.soap.search.GoogleSearch#doGetCachedPage(java.lang.String)
     */
    public byte[] doGetCachedPage( String arg0 ) throws GoogleSearchFault
    {
        throw new UnsupportedOperationException(
                "unsupported in logging searches" );
    }

    /**
     * Unsupported in logging searches.
     * 
     * @see com.google.soap.search.GoogleSearch#doSpellingSuggestion(java.lang.String)
     */
    public String doSpellingSuggestion( String arg0 ) throws GoogleSearchFault
    {
        throw new UnsupportedOperationException(
                "unsupported in logging searches" );
    }
}