package edu.ubc.cpsc.googlex;

import com.google.soap.search.*;

/**
 * A simplified version of the GoogleSearch class.
 * <p>
 * Please use SimpleGoogleSearch for this assignment rather than using
 * GoogleSearch directly. However, feel free to read through the code for
 * SimpleGoogleSearch to understand GoogleSearch better. We do NOT recommend
 * reading through the code for LoggedGoogleSearch and LoggingGoogleSearch, as
 * they are designed for simplicity of use rather than readability. (We would
 * have designed them differently if we wanted them to be generally useful and
 * readable.)
 * <p>
 * SimpleGoogleSearch simplifies access to GoogleSearch in three ways. First, it
 * hides GoogleSearch's use of arrays (which we'll learn about shortly!).
 * Second, it hides GoogleSearch's use of exceptions (which we won't discuss in
 * 111, although 211 does discuss them). Third, it provides access to Logged and
 * Logging versions of GoogleSearch to ease debugging and marking. Note: we will
 * mark this assignment using the logged mode of SimpleGoogleSearch and the log
 * we provided you! You should ensure that your code works in that mode!
 * <p>
 * How do "logging" and "logged" modes work? Well, if both are disabled,
 * SimpleGoogleSearch actually goes to Google and searches for the queries you
 * pose to it. In "logged" mode, however, SimpleGoogleSearch does not connect to
 * the internet at all (which means you can use logged mode even without a
 * network connection). Instead, it uses a file called test.log that we've
 * distributed to you. That file contains a collection of stored searches.
 * "Logged" mode simply accesses those stored searches, printing an error
 * message if a stored search matching the search you requested cannot be found.
 * You don't really need to know about "logging" mode, but in case you want to
 * use it.. Logging mode works just like normal mode except any searches
 * performed in logging mode are appended to the file test.log for future
 * reference in logged mode. (We created test.log by running the sample solution
 * in logging mode and performing some searches!)
 */
public class SimpleGoogleSearch
{
    /**
     * The default logging setting for new SimpleGoogleSearches.
     */
    private static boolean ourDefaultIsLogging;

    /**
     * The default logging setting for new SimpleGoogleSearches.
     */
    private static boolean ourDefaultIsLogged;

    /**
     * The GoogleSearch to use for searches. Initializing this to a
     * LoggedGoogleSearch or a LoggingGoogleSearch (using a mechanism called
     * polymorphism that we'll discuss at the end of the term) allows either
     * logging your searches as they're performed or working from logged
     * searches, respectively.
     */
    private final GoogleSearch mySearch;

    /**
     * The latest search result produced by a search.
     */
    private GoogleSearchResult myResult = null;

    /**
     * Create a SimpleGoogleSearch that uses the provided Google API key. The
     * resulting search will use the static default logging settings to
     * determine whether it is a true Google searcher or whether it logs
     * ("logging") or runs from a log ("logged").
     * 
     * @param key a valid Google API key (non-null)
     */
    public SimpleGoogleSearch( String key )
    {
        this( key, isDefaultLogging(), isDefaultLogged() );
    }

    /**
     * Create a SimpleGoogleSearch that uses the provided Google API key. If the
     * logging parameter is true, the resulting searcher will access the
     * internet directly and log any results for later use by a logged searcher.
     * If the logged parameter is true, the resulting searcher will NOT access
     * the internet directly, working instead from logs. If BOTH are true, the
     * logged parameter overrides the logging parameter. If neither is true, the
     * resulting searcher accesses the internet directly but neither logs nor
     * works from a log.
     * 
     * @param key a valid Google API key (non-null)
     * @param logging whether to create a logging searcher
     * @param logged whether to create a searcher that works from a log
     */
    public SimpleGoogleSearch( String key, boolean logging, boolean logged )
    {
        if( logged )
        {
            this.mySearch = new LoggedGoogleSearch();
        }
        else if( logging )
        {
            this.mySearch = new LoggingGoogleSearch();
        }
        else
        {
            this.mySearch = new GoogleSearch();
        }
        this.mySearch.setKey( key );
    }

    /**
     * Gets the default "logged" status.
     * @return Returns the default "logged" status.
     */
    public static boolean isDefaultLogged()
    {
        return SimpleGoogleSearch.ourDefaultIsLogged;
    }

    /**
     * Sets the default "logged" status.
     * @param isDefaultLogged The value to set the default logged status to.
     */
    public static void setIsDefaultLogged( boolean isDefaultLogged )
    {
        SimpleGoogleSearch.ourDefaultIsLogged = isDefaultLogged;
    }

    /**
     * Gets the default "logging" status.
     * @return Returns the default "logging" status.
     */
    public static boolean isDefaultLogging()
    {
        return SimpleGoogleSearch.ourDefaultIsLogging;
    }

    /**
     * Sets the default "logging" status.
     * @param isDefaultLogging The value to set the default logging status to.
     */
    public static void setIsDefaultLogging( boolean isDefaultLogging )
    {
        SimpleGoogleSearch.ourDefaultIsLogging = isDefaultLogging;
    }

    /**
     * Search for the given query, that is, a string you might type into Google
     * when performing a search.
     * 
     * @param queryString a query (non-null)
     * @return true if the search succeeds (i.e., encounters no errors, even if
     *         it doesn't return any results) and false otherwise
     */
    public boolean doSearch( String queryString )
    {
        this.mySearch.setQueryString( queryString );
        try
        {
            this.myResult = this.mySearch.doSearch();
            return true;
        }
        catch( GoogleSearchFault e )
        {
            System.out.println( "Search failed unexpectedly.  Error was: " + e );
            this.myResult = null;
            return false;
        }
    }

    /**
     * Get the number of hits the latest search produced.
     * 
     * @return the number of hits or -1 if there has not yet been a search or if
     *         the last search failed
     */
    public int getHitCount()
    {
        if( this.myResult == null )
            return -1;

        return this.myResult.getEstimatedTotalResultsCount();
    }

    /**
     * Gets the number of search results (i.e., snippets) from the latest
     * search.
     * 
     * @return the number of search results or 0 if there has not yet been a
     *         search or if the last search failed
     */
    public int getNumResults()
    {
        if( this.myResult == null )
            return 0;

        GoogleSearchResultElement[] results = this.myResult.getResultElements();
        return results.length;
    }

    /**
     * The snippet Google returned for the search result with the given index.
     * 
     * @param index a number between 0 and getNumResults - 1. NOTE THAT THIS
     *        STARTS AT ZERO!
     * @return the snippet at the given index
     */
    public String getSnippetAt( int index )
    {
        if( this.myResult == null )
            return null;

        GoogleSearchResultElement[] results = this.myResult.getResultElements();
        return results[ index ].getSnippet();
    }
}
